from setuptools import setup, find_packages

setup(
    name='appyx',
    version='0.0.10',
    description='...',
    url='https://gitlab.com/eryx/appyx',
    author='Eryx Team',
    author_email='info@eryx.co',
    license='MIT',
    packages=find_packages(),
    install_requires=[],
)
